<?php

/* newstyles.php
 *
 * This parses the extra comments that the new wikistylescopy.php adds to the
 * XML document, removes the attributes wikistyles inserted, and tries to
 * convert the styles to valid WikiBook markup.
 */

SDV($PdfAttributeStyles, array('bgcolor', 'align', 'id'));
SDV($PdfAttributeMap, array('background-color' => 'bgcolor', 
			    'text-align' => 'align',
                            'id' => 'id'));

/* settings for ApplyStyle tags */

SDV($imgTag,  'tbook:graphics');
SDV($aTag,    'tbook:url');
/* spanTag is not reset, as this is only used for creating new
 * span tags, which we wish to strip out anyway
 * SDV($spanTag, 'tbook:style');
 */

SDV($blockTags,
  'tbook:(?:group|tabular|row|cell|p|itemize|enumerate|description|item|term|h|blockquote|verbatim|hr)');

/* settings for wikistyles */

$ColorRGB = array(
	'black' => "#000000", 		'green' => "#008000",
	'silver' => "#C0C0C0", 		'lime' => "#00FF00",
	'gray' => "#808080", 		'olive' => "#808000",
	'white' => "#FFFFFF", 		'yellow' => "#FFFF00",
	'maroon' => "#800000", 		'navy' => "#000080",
	'red' => "#FF0000", 		'blue' => "#0000FF",
	'purple' => "#800080", 		'teal' => "#008080",
	'fuchsia' => "#FF00FF",		'aqua' => "#00FFFF" );
foreach ($ColorRGB as $k=>$v) {
  SDV($WikiStyle[$k]['color'],$k);
}

SDVA($WikiStyleApply,
     array(
	   'item' => 'tbook:item',
	   'list' => 'tbook:enumerate|tbook:itemize|tbook:description',
	   'div' => 'tbook:group|tbook:figure',
	   'img' => 'tbook:graphics',
	   'block' => 'tbook:p|tbook:blockquote|tbook:h|tbook:item|tbook:enumerate|tbook:itemize|tbook:figure|tbook:description|tbook:term|tbook:verbatim',
	   'p' => 'tbook:p')
     );

SDV($WikiStyleText, 'WikiStyle');
SDV($WikiStyleTagPrefix, "<!--$WikiStyleText-->");
SDV($WikiStyleAttrPrefix, "$WikiStyleText:");
SDV($WikiStyleTag, $WikiStyleTagPrefix.'<!--$1-->');

SDV($WikiStyle['lframe'], array('class' => 'lframe'));
SDV($WikiStyle['rframe'], array('class' => 'rframe'));
SDV($WikiStyle['cframe'], array('class' => 'cframe'));

SDV($WikiStyle['audience'], array('class' => 'mnote'));

function newStyles($document) {
  global $PdfAttributeStyles, $WikiStyleTagPrefix, $WikiStyleAttrPrefix,
    $PdfAttributeMap;

  $stack = array();

  $stylea = preg_split("/$WikiStyleTagPrefix/", $document);

  $out = '';

  $styles = array();
  $context = array();

  foreach($stylea as $str) {
    $new_styles_raw = array();
    while(preg_match('/^<!--(.+?)-->/', $str, $matches0)) {

      preg_match_all("/ *($WikiStyleAttrPrefix)?(.+?)=(\".+?\"|'.+?')/",
		     $matches0[1], $matches1_,
		     PREG_SET_ORDER);
      foreach($matches1_ as $matches1) {
	$key = $matches1[2];
	$value = $matches1[3];
	$value = substr($value, 1, strlen($value)-2); // strip quotes
	$new_styles_raw[] = array( 'name' => $key,
			       'value' => $value );
      }
      $str = substr($str, strlen($matches0[0]));
    }
    $new_styles = array();
    foreach($new_styles_raw as $style) {
      $name = $style['name'];
      $value = $style['value'];
      if ($name == 'class') {
	foreach(preg_split(',\s+,', $value) as $class) {
	  $new_styles['class.'.$class] = $class;
	}
      } elseif ($name == 'style') {  

	foreach(NewStylesParseCSS($value) as $sk => $sv) {
	  $new_styles[$sk] = $sv;
	}
      } elseif (in_array($name, $PdfAttributeStyles) ||
		isset($PdfAttributeMap[$name])) {
	$new_styles[$name] = $value;
      }
    }

    // <!--style--><!--a="b"-->... should be followed by a tag
    while(strlen($str) > 0) {
      // eat text
      if (preg_match("/^[^<]+/", $str, $matches)) {
	$out .= NewStylesWrapText($matches[0], $styles, $context);
	$str = substr($str, strlen($matches[0]));
      } elseif (preg_match("/^<!--.*?-->/", $str, $matches)) {
	$out .= $matches[0];
	$str = substr($str, strlen($matches[0]));
      } elseif (preg_match("/^<!.*?>/", $str, $matches)) {
	$out .= $matches[0];
	$str = substr($str, strlen($matches[0]));
      } elseif (preg_match("/^<(?!!)([^>]+)>/", $str, $matches)) {
	$tag = $matches[0];

	if (preg_match(",^</,", $tag)) {
	  // closing tag
	  $top = array_pop($stack);
	  $styles = $top['styles'];
	  $context = $top['context'];
	} elseif (preg_match(",<([a-z0-9:]+)(.*?)/>,", $tag, $tmatches)) {
	  $tagname = $tmatches[1];
	  $attrs = $tmatches[2];
	  $attrs = preg_replace(",(^|\\s+)(?:$WikiStyleAttrPrefix)[a-z]+=('.*?'|\".*?\"),"," ", $attrs);
	  
	  $tag = '<'.$tagname.$attrs.'/>';

	  if ($styles['display'] == 'none')
	    $tag = '';
	  // singleton tag
	  // ignore for now, but eat new_styles
	  $new_styles = array();
	} else {
	  // opening tag
	  array_push($stack, 
		     array('context' => $context, 'styles' => $styles));

	  foreach ($new_styles as $key => $value) {
	    $mkey = $key;
	    if (isset($PdfAttributeMap[$key])) {
	      $mkey = $PdfAttributeMap[$key];
	    }
	    $styles[$mkey] = $value;
	  }
	  $new_styles = array();

	  if (preg_match(",<([a-z0-9:]+)(.*?)>,", $tag, $tmatches)) {
	    $tagname = $tmatches[1];
	    $attrs = $tmatches[2];

	    $attrs = preg_replace(",(^|\\s+)(?:$WikiStyleAttrPrefix)[a-z]+=('.*?'|\".*?\"),"," ", $attrs);
	    $context[$tagname]++;
	    $newattrs = '';
	    if ($styles['display'] == 'none') {
              if ($tagname == 'tbook:group') {
                  $newattrs .= "class='comment'";
	          unset($styles['display']);
	      } else $tag = '';
	    } elseif ($tagname == 'tbook:p') {
	        if (isset($styles['id'])) {
	          $newattrs .= " id='".$styles['id']."'";
	          unset($styles['id']);
	        }	    
		if (in_array($styles['align'],array('left','center','right'))) {
	         $newattrs .= ' align="'.$styles['align'].'"';
	         unset($styles['align']);
	       }
	    } elseif ($tagname == 'tbook:figure') {
		if (($xs = isset($styles['class.lfloat']) ? 'lfloat' :
		    (isset($styles['class.rfloat']) ? 'rfloat' :
		    (isset($styles['class.lframe']) ? 'lframe' :
		    (isset($styles['class.rframe']) ? 'rframe' : '')))) != '') {
	          $newattrs .= " float='".((substr($xs, 0, 1) == 'l') ?
				       'left' : 'right')."'";
	          $newattrs .= (substr($xs, 1) == 'frame') ? " frame='on'" : ''; 
	          unset($styles['class.'.$xs]);
		} elseif (($xs = ($styles['align'] == 'center')  ? 'align' :
                      (isset($styles['class.cframe']) ? 'cframe' : '')) != '') {
	          $newattrs .= " style='center'"; 
                  if (substr($xs, 1) == 'frame') {
                      $newattrs .= " frame='on'";
                      unset($styles['class.'.$xs]);
	          } else unset($styles[$xs]);
		}
	        if (isset($styles['id'])) {
	          $newattrs .= " id='".$styles['id']."'";
	          unset($styles['id']);
	        }
	    } elseif ($tagname == 'tbook:group') {
	      $xs = isset($styles['class.frame']) ? 'frame' : '';

	      if (isset($styles['bgcolor'])) {
		$bgc = $styles['bgcolor'];
		$ngc = NewStylesTranslateColor($bgc);
		$newattrs .= " bgcolor='".NewStylesTranslateColor($styles['bgcolor'])."' ";
		unset($styles['bgcolor']);
	      }
	      if ($xs != '') { $newattrs .= " class='$xs'"; }
	    }
	    $tag = '<'.$tagname.$attrs.$newattrs.'>';
	  } else {
	    $out .= "<!--ERROR-->";
	  }
	}
	$out .= $tag;

	$str = substr($str, strlen($matches[0]));
      } else {
	echo "<!-- ERROR $str -->";
      }
    }
  }
  return $out;
}

function NewStylesTranslateColor($color) {
  global $ColorRGB;
  if (preg_match(',^#([0-9A-Fa-f])([0-9A-Fa-f])([0-9A-Fa-f])$,', $color, $matches)) {
    return '#'.$matches[1].$matches[1].$matches[2].$matches[2].$matches[3].$matches[3];
  } elseif (preg_match(',^#[0-9A-Fa-f]{6}$,', $color, $matches)) {
    return $color;
  } elseif (isset($ColorRGB[$color])) {
    return $ColorRGB[$color];
  }
  return '';
}

function NewStylesWrapText($text, $styles, $context) {
  global $ColorRGB;
  if ($styles['display'] == 'none')
    return "";
  if ($context['tbook:verbatim'] != 0) {
    return $text;
  }
  if (preg_match(',^\s*$,s', $text)) return $text;
  if (isset($styles['bgcolor']) && $context['tbook:url'] == 0 
      && ($xc=NewStylesTranslateColor($styles['bgcolor'])) != '') {
    $text = "<tbook:style bgcolor='$xc'>".$text."</tbook:style>";
  }
  
  if (isset($styles['color']) && $context['tbook:url'] == 0
      &&  ($xc=NewStylesTranslateColor($styles['color'])) != '') {
    $text = "<tbook:style color='$xc'>".$text."</tbook:style>";
  }

  if (isset($styles['class.mnote']))
    $text = "<tbook:mnote class='mnote'>".$text."</tbook:mnote>";

  return $text;

}

function NewStylesParseCSS($style) {
  $values = array();

  while($style != '') {
    if (preg_match(',^\;+,', $style, $matches)) {
      $style = substr($style, strlen($matches[0]));
    } elseif (preg_match(',^\s+,', $style, $matches)) {
      $style = substr($style, strlen($matches[0]));
    } elseif(preg_match(',^([^: ]+):([^\}\;]+),', $style, $matches)) {
      $key = trim($matches[1]);
      $value = trim($matches[2]);
      $values[$key] = $value;
      $style = substr($style, strlen($matches[0]));
    } else {
      return $values;
    }
  }
  return $values;
}




?>